import 'dart:async';

import 'package:flutter/material.dart';
import 'package:osam/domain/state/base_state.dart';
import 'package:osam/osam.dart';

part 'navigation_presenter.dart';
part 'navigation_state.g.dart';
part 'osam_navigator.dart';

void registerNavigationAdapters() {
  Hive.registerAdapter(NavigationStateAdapter());
  Hive.registerAdapter(TabNavigationStateAdapter());
}

@HiveType(typeId: 0)
// ignore: must_be_immutable
class NavigationState extends BaseState<NavigationState> {
  @HiveField(0)
  final List<String> routes;

  NavigationState(this.routes);

  StateStream<List<String>> get routesStream => propertyStream((_) => routes);

  void push(String route) => routes.add(route);

  void pushReplacement(String route) =>
      update((_) => routes.replaceRange(routes.length - 1, routes.length, [route]));

  void pushAndRemoveUntil(String route) => update((_) => routes
    ..clear()
    ..add(route));

  void pop() => update((_) => routes.removeLast());

  @override
  List<Object> get props => [routes];
}

@HiveType(typeId: 1)
// ignore: must_be_immutable
class TabNavigationState extends BaseState<TabNavigationState> {
  @HiveField(0)
  String tab;

  TabNavigationState(this.tab);

  StateStream<String> get tabStream => propertyStream((state) => state.tab);

  void changeTab(String tab) => tab = tab;

  @override
  List<String> get props => [tab];
}
