part of 'navigation_state.dart';

PageRoute _pageRoute({WidgetBuilder builder, RouteSettings settings, bool isMaterial = false}) =>
    isMaterial
        ? MaterialPageRoute(builder: builder, settings: settings)
        : CupertinoPageRoute(builder: builder, settings: settings);

class OsamNavigator<P extends NavigationPresenter<BaseState>> extends StatefulWidget {
  final Map<String, WidgetBuilder> _screens;
  final bool rootNavigator;
  final bool isMaterial;

  const OsamNavigator(this._screens, {Key key, this.rootNavigator = false, this.isMaterial = false})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<P>();
}

class _OsamNavigatorState<P extends NavigationPresenter<BaseState>> extends State<OsamNavigator>
    with AfterLayoutMixin {
  final List<String> routes = [];
  StreamSubscription<List<String>> routesSub;

  Map<String, WidgetBuilder> get screens => widget._screens;

  @override
  Widget build(BuildContext context) =>
      screens[context.presenter<P>().routesStream.initialData.first](context);

  void _silentPop(NavigationPresenter presenter) {
    if (presenter.routesStream.initialData.length > 1) {
      routes.removeLast();
      presenter.pop();
    }
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context, rootNavigator: widget.rootNavigator);
    final presenter = context.presenter<P>();
    final initialRoutes = presenter.routesStream.initialData;
    routes.addAll(initialRoutes);
    if (initialRoutes.length > 1) {
      final firstRoute = initialRoutes.first;
      navigator.pushReplacement(_pageRoute(
          builder: ((ctx) => screens[firstRoute])(context),
          settings: const RouteSettings(isInitialRoute: true),
          isMaterial: widget.isMaterial));
      final others = initialRoutes.sublist(1, initialRoutes.length);
      others.forEach((route) {
        navigator.push(
            _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
              ..popped.then((_) => _silentPop(presenter)));
      });
    }
    presenter._routesSub.onData((routes) {
      final routesDifferentNumber = routes.length - routes.length;
      if (routesDifferentNumber == 0 && routes.last != routes.last) {
        navigator.pushReplacement(_pageRoute(
            builder: ((ctx) {
              return screens[routes.last](context);
            }),
            isMaterial: widget.isMaterial)
          ..popped.then((_) => _silentPop(presenter)));
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = routes.sublist(routes.length, routes.length);
        pushRoutes.forEach((route) {
          navigator.push(
              _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
                ..popped.then((_) => _silentPop(presenter)));
        });
      } else if (routesDifferentNumber < 0) {
        routes.first != routes.first
            ? navigator.pushAndRemoveUntil(
                _pageRoute(
                    builder: ((ctx) {
                      return screens[routes.first](context);
                    }),
                    isMaterial: widget.isMaterial),
                (route) => false)
            : routes.sublist(routes.length, routes.length).forEach((route) async {
                navigator.pop();
              });
      }
      routes.clear();
      routes.addAll(routes);
    });
  }
}

class OsamTabNavigator<P extends TabNavigationPresenter<BaseState>> extends StatelessWidget {
  final Map<String, Widget> navigators;
  final Map<String, Widget> tabs;
  final bool isMaterial;
  final BoxDecoration tabDecoration;
  final _navigatorKeys = <String, GlobalKey<NavigatorState>>{};

  OsamTabNavigator({Key key, this.navigators, this.tabDecoration, this.tabs, this.isMaterial})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<P>();
    if (_navigatorKeys.isEmpty) {
      _navigatorKeys.addAll(navigators.map((k, v) => MapEntry(k, GlobalKey<NavigatorState>())));
    }
    return OsamBuilder<String>(
      key: const ValueKey('tab_ob'),
      stream: presenter.tabStream,
      builder: (ctx, snapshot) {
        return Stack(
          children: <Widget>[
            IndexedStack(
              index: navigators.keys.toList().indexOf(snapshot.data),
              children: navigators.keys.map((tab) {
                return _OsamNavigatorPage(
                  _navigatorKeys[tab],
                  navigators[tab],
                  isMaterial,
                  key: ValueKey(tab.toString() + '_onp'),
                );
              }).toList(),
            ),
            Positioned(
              bottom: 0,
              child: SafeArea(
                bottom: true,
                child: Container(
                  decoration: tabDecoration,
                  padding: const EdgeInsets.symmetric(vertical: 5),
                  constraints: BoxConstraints(
                    maxWidth: MediaQuery.of(context).size.width,
                    minHeight: 48,
                  ),
                  child: Row(
                    children: navigators.keys
                        .map((tab) => Expanded(
                              child: GestureDetector(
                                child: Container(
                                  child: tabs[tab],
                                  alignment: Alignment.center,
                                  color: Colors.transparent,
                                ),
                                onTap: () {
                                  if (snapshot.data != tab) {
                                    presenter.changeTab(tab);
                                  } else {
                                    // TODO(Daniel): add this
//                                    final currentPresenter = navigators[snapshot.data].presenter;
//                                    final routes = currentPresenter.routeStream.initialData;
//                                    if (routes.length > 1) {
//                                      currentPresenter.pushAndRemoveUntil(routes.first);
//                                    }
                                  }
                                },
                              ),
                            ))
                        .toList(),
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }
}

class _OsamNavigatorPage extends StatelessWidget {
  final GlobalKey<NavigatorState> navKey;
  final Widget child;
  final bool isMaterial;

  const _OsamNavigatorPage(this.navKey, this.child, this.isMaterial, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navKey,
      onGenerateRoute: (RouteSettings settings) => _pageRoute(
          settings: const RouteSettings(isInitialRoute: true),
          builder: (BuildContext context) => child,
          isMaterial: isMaterial),
    );
  }
}
