import 'package:hive/hive.dart';
import 'package:path_provider/path_provider.dart';

class PersistRepository<ST, US> {
  Box<ST> _persist;
  Box<US> _uiPersist;

  Future<void> initPersist(String appName) async {
    Hive.init((await getApplicationDocumentsDirectory()).path);
    _persist ??= await Hive.openBox(appName);
    _persist ??= await Hive.openBox(appName + 'navigation');
  }

  void save(ST state, US uiState) {
    _persist.put('appState', state);
    _uiPersist.put('uiState', uiState);
  }

  ST getState() => _persist.get('appState');

  US getUiState() => _uiPersist.get('uiState');

  void delete() {
    _persist.delete('appState');
    _uiPersist.delete('uiState');
  }

  void deleteFromDisk(String appName) {
    Hive.deleteBoxFromDisk(appName);
    Hive.deleteBoxFromDisk(appName + 'navigation');
  }

  void deleteHive() => Hive.deleteFromDisk();

  bool get isPersistEnabled => _persist == null ? false : true;
}
