part of '../domain/store/store.dart';

class StoreProvider extends InheritedWidget {
  StoreProvider({
    Key key,
    Widget child,
    @required this.store,
    this.persist,
    this.uiState,
  }) : super(
            key: key,
            child: (persist?.isPersistEnabled ?? false) ? _LifecycleWrapper(child: child) : child);

  final Store store;
  final BaseState uiState;
  final PersistRepository persist;

  static StoreProvider of(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType(aspect: StoreProvider);

  @override
  bool updateShouldNotify(StoreProvider oldWidget) => false;
}

class _LifecycleWrapper extends StatefulWidget {
  final Widget child;

  const _LifecycleWrapper({Key key, this.child}) : super(key: key);

  @override
  _LifecycleWrapperState createState() => _LifecycleWrapperState();
}

class _LifecycleWrapperState extends State<_LifecycleWrapper> with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused) {
      final storeProvider = StoreProvider.of(context);
      storeProvider.persist.save(storeProvider.store._state, storeProvider.uiState);
    }
  }
}

mixin _PresenterProviderSingleChildWidget on SingleChildWidget {}

class PresenterProvider<P extends Presenter<BaseState>> extends SingleChildStatelessWidget
    with _PresenterProviderSingleChildWidget {
  final Widget child;
  final P presenter;

  const PresenterProvider({Key key, this.child, this.presenter}) : super(key: key, child: child);

  static P of<P extends Presenter<BaseState>>(BuildContext context) =>
      Provider.of<P>(context, listen: false);

  @override
  Widget buildWithChild(BuildContext context, Widget child) {
    final storeProvider = StoreProvider.of(context);

    return Provider<P>(
      key: ValueKey('PresenterProvider: %${key.toString()}'),
      create: (_) {
        // ignore: invalid_use_of_protected_member
        presenter?.store = storeProvider.store;
        // ignore: invalid_use_of_protected_member
        presenter?.uiState = storeProvider?.uiState;
        presenter?.init();
        return presenter;
      },
      child: child,
      dispose: (ctx, presenter) => presenter?.dispose(),
    );
  }
}

class MultiPresenterProvider extends StatelessWidget {
  final List<SingleChildWidget> providers;
  final Widget child;

  const MultiPresenterProvider({Key key, @required this.providers, @required this.child})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: providers,
      key: ValueKey('PresenterMultiProvider: %${key.toString()}'),
      child: child,
    );
  }
}

extension PresenterProviderExtension on BuildContext {
  P presenter<P extends Presenter<BaseState>>() => PresenterProvider.of<P>(this);
}
